#include<iostream>
#include<fstream>
#include<vector>
#include<algorithm>
#include<string>
#include<random>
#include<numeric>
using namespace std;

mt19937 randgen;

int rand_range(int a, int b) {
    return randgen()%(b+1-a) + a;
}

int min(vector<int> &A) {
    return *min_element(A.begin(), A.end());
}

int max(vector<int> &A) {
    return *max_element(A.begin(), A.end());
}

int sum(vector<int> &A) {
    return accumulate(A.begin(), A.end(), 0);
}


void write_test(string filename, int K, vector<int>& A, vector<int>& B) {
    ofstream out(filename);
    out << A.size() << ' ' << B.size() << ' ' << K << '\n';
    
    for(int i=0; i<(int)A.size(); i++) {
        out << A[i] << " \n"[i+1 == (int)A.size()];
    }
    
    for(int i=0; i<(int)B.size(); i++) {
        out << B[i] << " \n"[i+1 == (int)B.size()];
    }
}

void write_custom(string filename, int K, vector<int> A, vector<int> B) {
    write_test(filename, K, A, B);
}

void one_choice_test(string filename, int n, int a, int m_b, int b, int k, int red_cnt, vector<int> extra, int to_add = 1) {
    vector<int> A, B = extra;
    for(int i=0; i<m_b; i++) {
        B.push_back(b);
    }
    for(int i=0; i<red_cnt; i++) {
        B.push_back(b+to_add);
    }
    random_shuffle(B.begin(), B.end());
    for(int i=0; i<n; i++) {
        A.push_back(a);
    }
    write_test(filename, k, A, B);
}

void random_test(string filename, int n, int m, int k, int a_min, int a_max, int b_min, int b_max, bool diagnose = false) {
    vector<int> A, B;
    int box_filled = 0;
    for(int i=0; i<n; i++) {
        A.push_back(rand_range(a_min, a_max));
    }
    for(int i=0; i<m; i++) {
        B.push_back(rand_range(b_min, b_max));
        box_filled += min(B.back(), n);
    }
    write_test(filename, k, A, B);
    
    if(diagnose) {
        cout << filename << '\n';
        cout << "K*N: " << k*n << '\n';
        cout << "min(A): " << min(A) << '\n';
        cout << "box_filled: " << box_filled << '\n';
        cout << "sum(A): " << sum(A) << '\n';
        cout << "sum(B): " << sum(B) << '\n';
    }
}


int main() {
    randgen.seed(17512351);
    
    // Subtask 1
    
    write_custom("tests/input3.txt", 1, {12, 43, 61, 11}, {130, 168}); // Choose smaller
    write_custom("tests/input4.txt", 1, {12, 43, 61, 11}, {98, 130}); // Choose larger
    write_custom("tests/input5.txt", 1, {61, 23, 23, 75, 12, 53}, {135, 221}); // Choose both due to sum
    write_custom("tests/input6.txt", 2, {2, 5, 4, 3, 7, 73, 53, 2, 7, 3, 4, 5, 8, 9, 5, 3}, {231, 274}); // Choose both due to K
    random_test("tests/input7.txt", 300, 2, 1, 1, 2, 250, 300); // Choose both, big N
    write_custom("tests/input8.txt", 2, {2, 2, 2, 2, 2, 2, 2, 3, 4, 5, 6, 7, 1, 2, 3}, {14, 200}); // Impossible, box_filled too small
    write_custom("tests/input9.txt", 2, {52, 5, 3, 61, 2, 4, 7, 1, 34, 12, 1, 5}, {200, 279}); // Impossible, contains low A
    write_custom("tests/input10.txt", 1, {63, 13, 53, 74, 23, 1, 2, 1, 5}, {123, 111}); // Impossible, too low total sum
    
    // Subtask 2
    
    random_test("tests/input11.txt", 8, 15, 10, 150, 300, 100, 300); // 6 freedom of choice
    write_custom("tests/input12.txt", 7, {300, 300, 300, 149}, {144, 143, 142, 141, 147, 148, 149, 150, 151, 152, 153, 300, 299, 298, 297}); //Manual one choice test
    random_test("tests/input13.txt", 15, 15, 15, 15, 15, 14, 17, true); // Impossible, box_filled too small
    random_test("tests/input14.txt", 30, 15, 5, 5, 7, 5, 23, true); // High K, high variance
    random_test("tests/input15.txt", 300, 15, 7, 7, 13, 200, 300); // -6 freedom, High N
    
    // Subtask 3
    
    one_choice_test("tests/input16.txt", 297, 168, 196, 252, 1, 50, {253, 254, 255, 256, 257, 258, 259, 260, 261, 262, 263, 264, 253, 253, 254});
    one_choice_test("tests/input17.txt", 297, 168, 197, 252, 1, 20, {255, 247, 246, 245, 244, 243, 242, 241, 240}, -5);
    random_test("tests/input18.txt", 290, 300, 1, 24, 300, 10, 300, true); // 6 freedom of choice
    random_test("tests/input19.txt", 281, 300, 1, 240, 300, 250, 300, true); // 20 freedom of choice
    random_test("tests/input20.txt", 300, 290, 1, 290, 300, 290, 300, true); // Impossible test, too low total sum
    one_choice_test("tests/input21.txt", 297, 168, 192, 252, 1, 0, {253, 251, 254, 250, 249, 272, 1, 1, 1, 2, 2, 3, 4, 5, 6, 154});
    
    
    // Subtask 4
    
    one_choice_test("tests/input22.txt", 26, 36, 24, 39, 30, 10, {27, 1, 2, 3, 40, 4}, -9);
    random_test("tests/input23.txt", 25, 40, 27, 27, 33, 1, 40, true); // 3 freedom of choice
    random_test("tests/input24.txt", 35, 40, 20, 20, 40, 25, 35, true); // 5 freedom of choice
    random_test("tests/input25.txt", 40, 40, 37, 37, 38, 35, 40, true); // Stress test
    random_test("tests/input26.txt", 40, 40, 30, 29, 40, 35, 40, true); // Impossible test, one A too small
    
    
    // Subtask 5
    
    one_choice_test("tests/input27.txt", 198, 252, 168, 297, 198, 30, {299, 299, 300, 1, 2, 3, 4, 1, 1, 1, 2, 1}); // Optimal all 297 and 298
    one_choice_test("tests/input28.txt", 198, 252, 168, 297, 189, 61, {200, 298, 298, 298, 298, 299, 299, 1, 1, 2, 2, 1, 72, 51}, -99); //Optimal 198, 100 and rest 297
    random_test("tests/input29.txt", 150, 160, 131, 134, 210, 51, 300, true); // 10 freedom of choice
    random_test("tests/input30.txt", 210, 244, 120, 167, 300, 140, 300, true); // 20 freedom of choice
    random_test("tests/input31.txt", 200, 250, 202, 250, 300, 260, 300, true); // 50 freedom of choice
    random_test("tests/input32.txt", 280, 300, 260, 260, 300, 260, 300, true); // Stress test
}

